# -*- coding: utf-8 -*-
# copyright: (c) 2020 by Jesse Johnson.
# license: MPL-2.0, see LICENSE for more details.
'''Provide exceptions.'''


class PromanWorkflowException(Exception):
    '''Provide base errors in PackageManager.'''
